//********************************************************
//
// Example Application of BRD_xxx functions for DAC216X400224X192 service.
//
// (C) InSys by Sklyarov A. Jan, 2008
//
//=********************************************************

#include	<conio.h>
#include	<stdio.h>
#include	<tchar.h>
#include	<ctype.h>
#include	<windows.h>
#include	<conio.h>
#include	<dos.h>
#include	<math.h>
#include	"brd.h"
#include	"ctrlstrm.h"
#include	"ctrlcmpsc.h"
#include	"CtrlDac.h"
#include	"CtrlDac216x400m.h"
#include	"ctrlsdram.h"

#define PI2   6.2831853071795864
const U16 PLD_CFG_TAG	= 0x0500; // tag of PLD
BRD_Handle g_hCaptSrv[10];
S32 g_nCaptSrv = 0;
S32	SamplesOnPeriod=10;
double g_OutRate=25000000.0;
__int64 g_OutBufSize;
S32 g_DacFifoSize;
S32	status;
int	g_NumChans;
U32 g_SdramWriteBufSize=1;
S32	*g_pbufSdram[6];
//------------------------------------------------------------------------
void DisplayError(S32 status, TCHAR* cmd_str);
S32	CaptureSrv(BRD_Handle handle,TCHAR* serviceName );
S32	SetParamSrv(void);
S32	WorkSrv(void);
S32	FifoOutputCPU( PVOID pSig, ULONG bBufSize);
S32	FifoOutputDMA(void);
S32	SdramWriteDMA(void);
S32	SdramOutToDAC(void);
S32	FifoOutputCycleDMA(void);
S32 FifoOutputCPUCycleFIFO( PVOID pSig, ULONG bBufSize,S32 cycle);
S32 CycleSdramOutput(S32 cycle);
void CorrectOutFreq(void);
void ProgramSyncDac(int nmod, int invdclki);
S32 CalcSignal(S32 *samples, S32 nsamples);
bool SetParamSDRAM(BRD_Handle hSrv);
bool  ReadIniFile( TCHAR *pFileName );
S32 SdramModulWriteDMA(int nmod);
S32 CalcSignalSDRAM(S32 *samples, S32 nsamples,double *phase);
//------------------------------------------------------------------------

//---------------------------------------------------------------------------
typedef struct 
{
	U32  WorkMode;
	U32  IsAlwaysWriteSdram;
	U32  SamplesPerChannel;
	U32  IsSystemMemory;
	U32  IsAlwaysLoadPld;
	U32	 MasterSlave;
	U32	 StartSource;
	U32	 StartInverse;
	U32  StartTrigger; 
	U32	 StopSource;
	U32	 StopInverse;
	float ThresholdComp0;
	float ThresholdComp1;
	TCHAR PldFileName[100];
	TCHAR ServiceName[100];
	TCHAR IniFileName[100];
}EXAM_PARAM;

EXAM_PARAM g_ExamParam=
{
	1,		// WorkMode
	1,		// IsAlwaysWriteSdram;
	1024,	// SamplesPerChannel
	0,		// IsSystemMemory
	0,		// IsAlwaysLoadPLD;
	0,		// MasterSlave;
	0,		// StartSource: source of start : 0-programm, 2-SDX(comparator 1)
	0,		// StartInverse: inverse signal start: 0-no, 1-yes    
	0,		// StartTrigger: start mode : 0- potential, 1-trigger
	0,		// StopSource: source of stop by trigger start mode
	0,		// StopInverse: inverse signal stop: 0-no, 1-yes
	0.1,	// ThresholdComp0 (v)
	0.1,	// ThresholdComp1 (v)
	{""},						// PldFileName
	{"DAC216X400M0"},			// ServiceName
	{"\\exam_dac216x400m.ini"}  //IniFileName
};
//---------------------------------------------------------------------------
typedef struct 
{
    U32  ClockSource;
    U32  ExtClockValue;
    U32  RefSource;
    U32  ExtRefValue;
	U32  MaskChannels;
	U32  DividerClk;
	U32	 AmpChan0;
	U32	 AmpChan1;
	U32	 OutFreq;
	U32	 SignalType;
    U32  SlaveClockSource;
    U32  Filter;
    U32  EnableAnalogQm;
    U32  INVDCLKI;
} DAC_PARAM;				// ADMDAC216x400M 

DAC_PARAM   g_DacParam =	// Read from ini-file
{
    0,			// ClockSource: 0-internal, 1-external
    200000000,	// ExtClockValue (Hz) 
    0,			// RefSource: 0-internal, 1-external
    10000000,	// ExtRefValue (Hz) 
    1,			// ChannelsMask: 1,3 
 	1,			// DividerClk: 1,2,4,8 or 16
	32000,		// AmpChan0: 0-32767
	32000,		// AmpChan1: 0-32767 
	10000000,	// OutFreq (Hz)
	0,			// SignalType: 0 - sin, 1 -meandr
    1,			// SlaveClockSource : 0-internal, 1-external;
    4,			// Filter;
    0,			// EnableAnalogQm;
    0,			// INVDCLKI;

};
//---------------------------------------------------------------------------

//=**************************** ReadIniPrepare ******************************
//=***************************************************************************
typedef struct {char section[25]; int format;void *ptr; char title[25]; } TReadIniEntry;
static TReadIniEntry    iniEntry[] =
{
	{"Option", 0, (void*)&g_ExamParam.WorkMode, "WorkMode" },
	{"Option", 0, (void*)&g_ExamParam.IsAlwaysWriteSdram, "IsAlwaysWriteSdram" },
	{"Option", 0, (void*)&g_ExamParam.SamplesPerChannel, "SamplesPerChannel" },
	{"Option", 0, (void*)&g_ExamParam.IsSystemMemory, "IsSystemMemory" },
	{"Option", 0, (void*)&g_SdramWriteBufSize, "SdramWriteBufSize" },
	{"Option", 3, (void*)&g_ExamParam.PldFileName, "PldFileName" },
	{"Option", 0, (void*)&g_ExamParam.IsAlwaysLoadPld, "IsAlwaysLoadPld" },
	{"Option", 0, (void*)&g_ExamParam.MasterSlave, "MasterSlave" },
    {"Option", 0, (void*)&g_ExamParam.StartSource,"StartSource" },
    {"Option", 0, (void*)&g_ExamParam.StartInverse,"StartInverse" },
    {"Option", 0, (void*)&g_ExamParam.StartTrigger,"StartTrigger" },
    {"Option", 0, (void*)&g_ExamParam.StopSource,"StopSource" },
    {"Option", 0, (void*)&g_ExamParam.StopInverse,"StopInverse" },
    {"Option", 1, (void*)&g_ExamParam.ThresholdComp0,"ThresholdComp0" },
    {"Option", 1, (void*)&g_ExamParam.ThresholdComp1,"ThresholdComp1" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.ClockSource,"ClockSource" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.ExtClockValue,"ExtClockValue" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.RefSource,"RefSource" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.ExtRefValue,"ExtRefValue" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.MaskChannels,"MaskChannels" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.DividerClk,"DividerClk" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.AmpChan0,"AmpChan0" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.AmpChan1,"AmpChan1" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.OutFreq,"OutFreq" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.SlaveClockSource,"SlaveClockSource" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.SignalType,"SignalType" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.Filter,"Filter" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.EnableAnalogQm,"EnableAnalogQm" },
    {"device0_dac216x400m0", 0, (void*)&g_DacParam.INVDCLKI,"INVDCLKI" },
};
//---------------------------------------------------------------------------

//=****************************** main **************************************
//=**************************************************************************
void main( int argc, char *argv[] )
{
	S32		status;
	if(!ReadIniFile(g_ExamParam.IniFileName ))
	{
		printf("Can't open file: %s !\n",g_ExamParam.IniFileName);
		printf("Press any key for leaving program...\n");
		_getch();
		_exit(0);
	}
	g_SdramWriteBufSize *=1024;
	BRD_displayMode(BRDdm_VISIBLE | BRDdm_CONSOLE); //     :     
	
	S32	DevNum;
	status = BRD_init("brd.ini", &DevNum); //  

	//   LID (   )
	BRD_LidList lidList;
	lidList.item = 10;
	lidList.pLID = new U32[10];
	status = BRD_lidList(lidList.pLID, lidList.item, &lidList.itemReal);

	BRD_Info	info;
	info.size = sizeof(info);
	BRD_Handle handle[10];

	for(ULONG iDev = 0; iDev < lidList.itemReal; iDev++)
	{
		BRD_getInfo(lidList.pLID[iDev], &info); //    
		printf("Board %d:  %s pid = %d\n",iDev,info.name,info.pid);
		printf("   DevID = 0x%x, RevID = 0x%x, Bus = %d, Dev= %d, Slot= %d.\n", 
			    info.boardType >> 16, info.boardType & 0xff, info.bus, info.dev, info.slot);
		handle[iDev] = BRD_open(lidList.pLID[iDev], BRDopen_SHARED, NULL); //     
		if(handle[iDev] <=0)printf("   Open Error!\n");
		if(handle[iDev] > 0)
		{
			BRD_PuList PuList[5];
			U32 ItemReal;
			status = BRD_puList(handle[iDev], PuList, 5, &ItemReal); //    
			if(ItemReal > 5)
			{
				ItemReal=5; 
				printf("   Number of programable unit >5 !\n");
			}
			for(U32 j = 0; j < ItemReal; j++)
			{
				printf("   Prog unit %d: %s, Id = %d, Code = %x, Attr = %x.\n",
									j, PuList[j].puDescription, PuList[j].puId, PuList[j].puCode, PuList[j].puAttr);
				
				if(PuList[j].puCode == PLD_CFG_TAG)
				{ //   
						U32	PldState;
						status = BRD_puState(handle[iDev], PuList[j].puId, &PldState);
						printf("   PLD State = %d\n", PldState);
						if(PldState==0 || g_ExamParam.IsAlwaysLoadPld==1)
						{ //  
					 		printf("   Loading PLD -> %s ...",g_ExamParam.PldFileName);
							status = BRD_puLoad(handle[iDev], PuList[j].puId, g_ExamParam.PldFileName, &PldState);
							if(BRD_errcmp(status, BRDerr_OK))
							{	
								printf(" OK!\n");
								PldState=1;
							}
							else 
							{	
								printf("  Error! \n");
								PldState=0;
							}
					    }
						if(PldState)CaptureSrv(handle[iDev],g_ExamParam.ServiceName);		
		         }
             }
         }
	}
	if(g_nCaptSrv>0)
	{
		SetParamSrv();
		WorkSrv();
	}else
	{
		printf("\n   No Service!\n");
	}
	for(int i = 0; i < g_nCaptSrv; i++)
		status = BRD_release(g_hCaptSrv[i], 0);

	for(ULONG iDev = 0; iDev < lidList.itemReal; iDev++)
		status = BRD_close(handle[iDev]); //   
	printf("BRD_close: OK\n");
	status = BRD_cleanup();
	printf("BRD_cleanup: OK\n");
	printf("Press any key for leaving program...\n");
	_getch();
}
// End of main
//=******************* DisplayError ***********************
//=********************************************************

void DisplayError(S32 status, TCHAR* cmd_str)
{
	S32	real_status = BRD_errext(status);
	TCHAR msg[255];
	switch(real_status)
	{
	case BRDerr_OK:
		sprintf_s(msg, "%s: BRDerr_OK\n", cmd_str);
		break;
	case BRDerr_BAD_MODE:
		sprintf_s(msg, "%s: BRDerr_BAD_MODE\n", cmd_str);
		break;
	case BRDerr_INSUFFICIENT_SERVICES:
		sprintf_s(msg, "%s: BRDerr_INSUFFICIENT_SERVICES\n", cmd_str);
		break; 
	case	BRDerr_NOT_ACTION:
		sprintf_s(msg, "%s:  BRDerr_NOT_ACTION\n", cmd_str);
		break;
	default:
		sprintf_s(msg, "%s: Unknown error\n", cmd_str);
		break;
	}
	printf(msg);
}
//=******************* Capture Srv ************************
//=********************************************************
S32 CaptureSrv(BRD_Handle handle,TCHAR* serviceName )
{
	S32		status;
	U32 mode = BRDcapt_EXCLUSIVE;
	U32	ItemReal;
	BRD_ServList srvList[10];
	status = BRD_serviceList(handle, 0, srvList, 10, &ItemReal);
	for(U32 j = 0; j < ItemReal; j++)
	{
		printf("   Srv %d:  %s, Attr = %X.\n",j, srvList[j].name, srvList[j].attr);
	}
    for(int iSrv = 0; iSrv < ItemReal; iSrv++)
    {			
		if(!strcmp(srvList[iSrv].name,g_ExamParam.ServiceName))
		{
		   printf("   Capture %s mode: EXCLUSIVE...",g_ExamParam.ServiceName); 
		   BRD_Handle hSrv = BRD_capture(handle, 0, &mode,srvList[iSrv].name, 10000);
			if(hSrv > 0)
			{
				g_hCaptSrv[g_nCaptSrv++]=hSrv;
				printf("OK!\n");

			}else
			{
				printf("Error!\n");
			}
		}
    }	
	return status;
}
//=**************************************************************************
//					    
//=**************************************************************************
S32 SetParamSrv(void)
{

	S32		status;
	U32	master[6]={0,0,0,0,0,0};
	ULONG maskChannels = g_DacParam.MaskChannels; 
	BRD_ClkRefMode clkRefMode;
	clkRefMode.srcClk=g_DacParam.ClockSource;
	clkRefMode.srcRef=g_DacParam.RefSource;
	clkRefMode.valueClk=g_DacParam.ExtClockValue;
	clkRefMode.valueRef=g_DacParam.ExtRefValue;

	BRD_DivClk divClk;
	divClk.divClk=g_DacParam.DividerClk;
	divClk.outRate=0;

	BRD_StartMode  startMode;
	startMode.startSrc= g_ExamParam.StartSource;
	startMode.startInv= g_ExamParam.StartInverse;
	startMode.trigOn= g_ExamParam.StartTrigger;
	startMode.trigStopSrc= 0;
 	startMode.stopInv= g_ExamParam.StopInverse;
	if((g_ExamParam.WorkMode==4) || (g_ExamParam.WorkMode==6))startMode.reStartMode=1;//Restart !!
	else startMode.reStartMode=0; 

	BRD_CmpSC cmp_sc;
	cmp_sc.src= 0;
	cmp_sc.thr[0]=g_ExamParam.ThresholdComp0;
	cmp_sc.thr[1]=g_ExamParam.ThresholdComp1;

	status = BRD_ctrl(g_hCaptSrv[0], 0, BRDctrl_CMPSC_SET, &cmp_sc);
	if(!BRD_errcmp(status, BRDerr_OK)) DisplayError(status, "BRDctrl_CMPSC_SET");

	if(g_nCaptSrv==1)master[0]=1; //single
	else
	{
		if(g_ExamParam.MasterSlave==1)master[0]=2; // master/slave
		else for(int j=0;j<g_nCaptSrv;j++)master[j]=1;

	}
	for(int i=0;i<g_nCaptSrv;i++)
	{
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETMASTER, &master[i]);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETMASTER:     master = %d\n", master[i]); 
		else
			DisplayError(status, "BRDctrl_DAC_SETMASTER");
//--------------------------------------------------------------------------------------------
		if(i==0)
		{
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETCLKREFMODE, &clkRefMode);
			if(BRD_errcmp(status, BRDerr_OK))
			{
				printf("BRDctrl_DAC_SETCLKREFMODE: srcClk= %d  valueClk = %.0f Hz\n",clkRefMode.srcClk,clkRefMode.valueClk);
				printf("                           srcRef= %d  valueRef = %.0f Hz\n",clkRefMode.srcRef,clkRefMode.valueRef);
			}
			else
				DisplayError(status, "BRDctrl_DAC_SETCLKREFMODE");
		}
//--------------------------------------------------------------------------------------------
		if(i!=0)
		{	
			if(g_ExamParam.MasterSlave==1) clkRefMode.srcClk=g_DacParam.SlaveClockSource;

			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETCLKREFMODE, &clkRefMode);
			if(BRD_errcmp(status, BRDerr_OK))
			{
				printf("BRDctrl_DAC_SETCLKREFMODE: srcClk= %d  valueClk = %.0f Hz\n",clkRefMode.srcClk,clkRefMode.valueClk);
				printf("                           srcRef= %d  valueRef = %.0f Hz\n",clkRefMode.srcRef,clkRefMode.valueRef);
			}
			else
				DisplayError(status, "BRDctrl_DAC_SETCLKREFMODE");
		}
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETCHANMASK, &maskChannels);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETCHANMASK:   maskChannels = %d\n", maskChannels);
		else
			DisplayError(status, "BRDctrl_DAC_SETCHANMASK");
		divClk.outRate=0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETDIVCLK, &divClk);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETDIVCLK:     div = %d outRate = %.1f Hz\n", divClk.divClk,divClk.outRate);
		else
			DisplayError(status, "BRDctrl_DAC_SETDIVCLK");
		g_OutRate=divClk.outRate;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETSTARTMODE, &startMode);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETSTARTMODE:  src = %d inv = %d  trig = %d  thresh = %.1fV restart = %d\n",
		startMode.startSrc,startMode.startInv,startMode.trigOn,g_ExamParam.ThresholdComp1,startMode.reStartMode);
		else
			DisplayError(status, "BRDctrl_DAC_SETSTARTMODE");
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_CMPSC_SET, &cmp_sc);

		BRD_DacSpec setAnalogQm;
		setAnalogQm.command=DAC216X400Mcmd_SETANALOGQM;
		setAnalogQm.arg= (PVOID)&g_DacParam.EnableAnalogQm; 
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETSPECIFIC,  &setAnalogQm); /// 
																					//  

	}
		return 1;
}
//*************************************************************************************
//			 
//*************************************************************************************
S32 WorkSrv(void)
{
	S32	status;
	S32	nblock=0;
	U32 tetrad;
	ULONG source;
	ULONG Enable;

	g_OutBufSize=g_ExamParam.SamplesPerChannel*2;	
	g_NumChans=1;
	if(g_DacParam.MaskChannels==3)
	{
		g_OutBufSize *=2;
		g_NumChans=2;
	}
	BRD_DacCfg dac_cfg;
	status = BRD_ctrl(g_hCaptSrv[0], 0, BRDctrl_DAC_GETCFG, &dac_cfg);
	g_DacFifoSize = dac_cfg.FifoSize; //   FIFO  
	printf("\nDacFifoSize = %d bytes\n",g_DacFifoSize);
//-------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==0)	//    (  )
	{
		printf("WorkMode: -- One time  CPU (FIFO) -- \n");
		printf("\n");
		int size=g_OutBufSize/16;
		g_OutBufSize=size*16;
		g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
		if(g_OutBufSize>g_DacFifoSize)
		{
			g_OutBufSize=g_DacFifoSize;
			g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
			printf("OutBufSize > DacFifoSize !\n");
		}
		printf("SamplesPerChannel = %d   ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		int *pSig = new int[g_OutBufSize];
		CalcSignal(pSig, g_ExamParam.SamplesPerChannel);
		printf("Press any key for stop!\n");
		while(!_kbhit())
		{
			FifoOutputCPU(pSig, g_OutBufSize );
			nblock++;
			printf("\rBlock %d",nblock);

		}
		printf("\n");
		delete pSig;
		_getch();
		return 1;
	}
//-------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==1)	//      (  )
	{
		printf("WorkMode: -- One time  DMA (FIFO) -- \n");
		printf("\n");
		printf("SamplesPerChannel = %d  ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		BRDctrl_StreamCBufAlloc buf_dscr[6];
		for(int i=0;i<g_nCaptSrv;i++)
		{
			buf_dscr[i].dir = BRDstrm_DIR_OUT;
			buf_dscr[i].isCont = g_ExamParam.IsSystemMemory; 
			buf_dscr[i].blkNum = 1;
			buf_dscr[i].blkSize = g_OutBufSize;
			buf_dscr[i].ppBlk = new PVOID[buf_dscr[i].blkNum];
			status = BRD_ctrl(g_hCaptSrv[i],0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr[i]);

			source = 0;		//      FIFO 
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETSOURCE, &source);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_GETSRCSTREAM, &tetrad);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_SETSRC, &tetrad);
			CalcSignal((S32*)buf_dscr[i].ppBlk[0], g_ExamParam.SamplesPerChannel);
		}
		printf("Press any key for stop!\n");
		while(!_kbhit())
		{
			if(FifoOutputDMA()==0)break;
			nblock++;
			printf("\rBlock %d",nblock);

		}
		printf("\n");
		for(int i=0;i<g_nCaptSrv;i++)
		{
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr[i]);
			delete[] buf_dscr[i].ppBlk;
		}
		_getch();
		return 1;
	}
	//---------------------------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==2)	//     
	{
		printf("WorkMode: -- Cycle DMA -- \n");
		printf("\n");
		CorrectOutFreq();
		printf("SamplesPerChannel = %d   ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		BRDctrl_StreamCBufAlloc buf_dscr[6];
		for(int i=0;i<g_nCaptSrv;i++)
		{
			buf_dscr[i].dir = BRDstrm_DIR_OUT;
			buf_dscr[i].isCont = g_ExamParam.IsSystemMemory; 
			buf_dscr[i].blkNum = 1;
			buf_dscr[i].blkSize = g_OutBufSize;
			buf_dscr[i].ppBlk = new PVOID[buf_dscr[i].blkNum];
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr[i]);
			CalcSignal((S32*)buf_dscr[i].ppBlk[0], g_ExamParam.SamplesPerChannel);
		}
		FifoOutputCycleDMA();
	    printf("Press any key for stop!\n");
		Enable=0;
		for(int i=0;i<g_nCaptSrv;i++)
		{
			status = BRD_ctrl(g_hCaptSrv[0], 0, BRDctrl_DAC_ENABLE,  &Enable); //  	
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr[i]);
			delete[] buf_dscr[i].ppBlk;
		}
		printf("\n");
		return 1;

	}
	//-------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==3)	//    FIFO	
	{
		printf("WorkMode: -- Cycle FIFO -- \n");
		printf("\n");
		int size=g_OutBufSize/16;
		g_OutBufSize=size*16;
		g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
		if(g_OutBufSize>g_DacFifoSize)
		{
			g_OutBufSize=g_DacFifoSize;
			g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
			printf("OutBufSize > DacFifoSize !\n");
		}
		int *pSig = new int[g_OutBufSize];
		CorrectOutFreq();
		printf("SamplesPerChannel = %d   ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		CalcSignal(pSig, g_ExamParam.SamplesPerChannel);
		FifoOutputCPUCycleFIFO(pSig, g_OutBufSize,1);
		printf("Press any key for stop!\n");
		while(!_kbhit());
		Enable = 0;
		
		for(int i=0;i<g_nCaptSrv;i++)
		{
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
		}
		printf("\n");
		delete pSig;
		_getch();
		return 1;

	}
//-------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==4)	//   FIFO   
	{
		printf("WorkMode: -- Restart FIFO -- \n");
		printf("\n");
		int size=g_OutBufSize/16;
		g_OutBufSize=size*16;
		g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
		if(g_OutBufSize>g_DacFifoSize)
		{
			g_OutBufSize=g_DacFifoSize;
			g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
			printf("OutBufSize > DacFifoSize !\n");
		}
		int *pSig = new int[g_OutBufSize];
		printf("SamplesPerChannel = %d   ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		CalcSignal(pSig, g_ExamParam.SamplesPerChannel);
		printf("Press any key for stop!\n");
		FifoOutputCPUCycleFIFO(pSig, g_OutBufSize,0 );
		while(!_kbhit());
		Enable = 0;
		for(int i=0;i<g_nCaptSrv;i++)
		{
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
		}
		printf("\n");
		delete pSig;
		_getch();
		return 1;

	}
//-------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==5)	// One time DMA (SDRAM)( !)
	{
		printf("WorkMode: -- One time  DMA (SDRAM) -- \n");
		printf("\n");
//		int size=g_OutBufSize/1024;
		_int64 size=g_OutBufSize/(g_SdramWriteBufSize);
		if(size==0)size=1;
		g_OutBufSize=size*g_SdramWriteBufSize;
		for(int i=0;i<g_nCaptSrv;i++)
		{
			if(!SetParamSDRAM(g_hCaptSrv[i]))
			{
				printf("\n SDRAM is absent ! \n");
				return 1;
			}
		}
		g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
		printf("SamplesPerChannel = %d   ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		BRDctrl_StreamCBufAlloc buf_dscr[6];
		for(int i=0;i<g_nCaptSrv;i++)
		{
			buf_dscr[i].dir = BRDstrm_DIR_OUT;
			buf_dscr[i].isCont = g_ExamParam.IsSystemMemory; 
			buf_dscr[i].blkNum = 1;
			buf_dscr[i].blkSize = g_SdramWriteBufSize;
			buf_dscr[i].ppBlk = new PVOID[buf_dscr[i].blkNum];
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr[i]);

 			source = 2;		//  data from  SDRAM 
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETSOURCE, &source);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_GETSRCSTREAM, &tetrad);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_SETSRC, &tetrad);
			g_pbufSdram[i]=(S32*)buf_dscr[i].ppBlk[0];
		}

			printf("Wait...\n");
			if(SdramWriteDMA()==0)goto end;
			printf("Press any key for stop!\n");
	    while(!_kbhit())
	   {	
		
			if(g_ExamParam.IsAlwaysWriteSdram)
			{
				if(SdramWriteDMA()==0)goto end;
			}
			SdramOutToDAC();
			nblock++; 
			printf("\rBlock %d",nblock);
		}
end:	printf("\n");
		for(int i=0;i<g_nCaptSrv;i++)
		{
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr[i]);
			delete[] buf_dscr[i].ppBlk;
		}
		_getch();
		return 1;
	}
//-----------------------------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==6)	// Restart SDRAM ( !)
	{
		printf("WorkMode: -- Restart SDRAM -- \n");
		printf("\n");
		_int64 size=g_OutBufSize/(g_SdramWriteBufSize);
		if(size==0)size=1;
		g_OutBufSize=size*g_SdramWriteBufSize;
		for(int i=0;i<g_nCaptSrv;i++)
		{
			if(!SetParamSDRAM(g_hCaptSrv[i]))
			{
				printf("\n SDRAM is absent ! \n");
				return 1;
			}
		}
 
		g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
		printf("SamplesPerChannel = %d   ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		BRDctrl_StreamCBufAlloc buf_dscr[6];
		for(int i=0;i<g_nCaptSrv;i++)
		{
			SetParamSDRAM(g_hCaptSrv[i]);
			buf_dscr[i].dir = BRDstrm_DIR_OUT;
			buf_dscr[i].isCont = g_ExamParam.IsSystemMemory; 
			buf_dscr[i].blkNum = 1;
			buf_dscr[i].blkSize =  g_SdramWriteBufSize;
			buf_dscr[i].ppBlk = new PVOID[buf_dscr[i].blkNum];
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr[i]);
			g_pbufSdram[i]=(S32*)buf_dscr[i].ppBlk[0];
			source = 2;		//  data from  SDRAM 
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETSOURCE, &source);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_GETSRCSTREAM, &tetrad);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_SETSRC, &tetrad);
		}
	    SdramWriteDMA();
		CycleSdramOutput(0);
	    printf("Press any key for stop!\n");
	    while(!_kbhit());
		for(int i=0;i<g_nCaptSrv;i++)
		{
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr[i]);
			Enable = 0;
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable);   //   
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_ENABLE, &Enable); //    SDRAM
			delete[] buf_dscr[i].ppBlk;
		}
		_getch();
		return 1;
	}
//-------------------------------------------------------------------------
	if(g_ExamParam.WorkMode==7)	// Cycle SDRAM)( !)
	{
		printf("WorkMode: -- Cycle SDRAM -- \n");
		printf("\n");
		_int64 size=g_OutBufSize/(g_SdramWriteBufSize);
		if(size==0)size=1;
		g_OutBufSize=size*g_SdramWriteBufSize;
		for(int i=0;i<g_nCaptSrv;i++)
		{
			if(!SetParamSDRAM(g_hCaptSrv[i]))
			{
				printf("\n SDRAM is absent ! \n");
				return 1;
			}
		}
		g_ExamParam.SamplesPerChannel=g_OutBufSize/g_NumChans/2;
		CorrectOutFreq();
		printf("SamplesPerChannel = %d   ",g_ExamParam.SamplesPerChannel);
		printf("OutFreq = %d Hz\n",g_DacParam.OutFreq);
		BRDctrl_StreamCBufAlloc buf_dscr[6];
		for(int i=0;i<g_nCaptSrv;i++)
		{
			buf_dscr[i].dir = BRDstrm_DIR_OUT;
			buf_dscr[i].isCont = g_ExamParam.IsSystemMemory; 
			buf_dscr[i].blkNum = 1;
			buf_dscr[i].blkSize = g_SdramWriteBufSize;
			buf_dscr[i].ppBlk = new PVOID[buf_dscr[i].blkNum];
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr[i]);
			g_pbufSdram[i]=(S32*)buf_dscr[i].ppBlk[0];
			source = 2;		//  data from  SDRAM 
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETSOURCE, &source);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_GETSRCSTREAM, &tetrad);
			BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_SETSRC, &tetrad);
			//CalcSignal((S32*)buf_dscr[i].ppBlk[0], g_ExamParam.SamplesPerChannel);
		}
		printf("Wait...\n");
		SdramWriteDMA();
		CycleSdramOutput(1);
		printf("Press any key for stop!\n");
	    while(!_kbhit());
		Enable = 0;
		for(int i=0;i<g_nCaptSrv;i++)
		{
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr[i]);
			Enable = 0;
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable);   //   
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_ENABLE, &Enable); //    SDRAM
			delete[] buf_dscr[i].ppBlk;
		}
		_getch();
		return 1;
	}
	return 1;
}
//-------------------------------------------------------------------------
//*********************************************************************************************
//		         
//*********************************************************************************************
S32 CalcSignal(S32 *samples, S32 nsamples)
{
	S32 hi,low;
	double delta=PI2*(double)g_DacParam.OutFreq/g_OutRate;

	//------------------------------------------------------
	if(g_DacParam.SignalType==0 || g_DacParam.SignalType==1)	//0-sin, 1- sin & cos
	{
		if(g_DacParam.MaskChannels==1)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				low=(S32)(g_DacParam.AmpChan0*sin(2*i*delta));
				hi=(S32)(g_DacParam.AmpChan0*sin((2*i+1)*delta));
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
			}
		}
		if(g_DacParam.MaskChannels==2)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				low=(S32)(g_DacParam.AmpChan1*sin(2*i*delta));
				hi=(S32)(g_DacParam.AmpChan1*sin((2*i+1)*delta));
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
			}
		}
		if(g_DacParam.MaskChannels==3)
		{
			if(g_DacParam.SignalType==0)
			{
				for(int i=0;i<nsamples;i++)
				{
					low=(S32)(g_DacParam.AmpChan0*sin(i*delta));
					hi=(S32)(g_DacParam.AmpChan1*sin(i*delta));
					samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
				}	
			}
			if(g_DacParam.SignalType==1)
			{
				for(int i=0;i<nsamples;i++)
				{
					low=(S32)(g_DacParam.AmpChan0*sin(i*delta));
					hi=(S32)(g_DacParam.AmpChan1*cos(i*delta));
					samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
				}	
			}

		}
	}
//-------------------------------------------------------------------------------
	if(g_DacParam.SignalType==2)	//square
	{
		float amp,amp1;
		if(g_DacParam.MaskChannels==1)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				if(sin(2*i*delta)>=0)amp=1; else amp=-1;
				if(sin((2*i+1)*delta)>=0)amp1=1; else amp1=-1;
				low=(S32)(g_DacParam.AmpChan0*amp);
				hi=(S32)(g_DacParam.AmpChan0*amp1);
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);

			}
		}
		if(g_DacParam.MaskChannels==2)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				if(sin(2*i*delta)>=0)amp=1; else amp=-1;
				if(sin((2*i+1)*delta)>=0)amp1=1; else amp1=-1;
				low=(S32)(g_DacParam.AmpChan1*amp);
				hi=(S32)(g_DacParam.AmpChan1*amp1);
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
			}
		}
		if(g_DacParam.MaskChannels==3)
		{
			for(int i=0;i<nsamples;i++)
			{
				if(sin(i*delta)>=0)amp=1; else amp=-1;
				low=(S32)(g_DacParam.AmpChan0*amp);
				hi=(S32)(g_DacParam.AmpChan1*amp);
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
			}
		}
	}
//------------------------------------------------------------------------------------
	if(g_DacParam.SignalType==2)	//square + Filter
	{	
		for(int	i=0;i<nsamples;i++)
		{
		
			low=0;
			hi=0;
			for(int j=0;j<g_DacParam.Filter;j++)
			{
				 low +=((samples[i+j]<<16)>>16)/g_DacParam.Filter;
				 hi +=(samples[i+j]>>16) /g_DacParam.Filter;
			}
			 samples[i]=(0xffff & low) | (0xffff0000 & (hi<<16));
		}
	}
return 1;

}
//*********************************************************************************************
//		         
//*********************************************************************************************
S32 CalcSignalSDRAM(S32 *samples, S32 nsamples,double *phase)
{
	S32 hi,low;
	double delta=PI2*(double)g_DacParam.OutFreq/g_OutRate;
	double ph=*phase;
	//------------------------------------------------------
	if(g_DacParam.SignalType==0 || g_DacParam.SignalType==1)	//0-sin, 1- sin & cos
	{
		if(g_DacParam.MaskChannels==1)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				low=(S32)(g_DacParam.AmpChan0*sin(ph+(2*i*delta)));
				hi=(S32)(g_DacParam.AmpChan0*sin(ph+(2*i+1)*delta));
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
				if(i==(nsamples/2-1))*phase=ph+(2*i+2)*delta;
			}
		}
		if(g_DacParam.MaskChannels==2)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				low=(S32)(g_DacParam.AmpChan1*sin(ph+(2*i*delta)));
				hi=(S32)(g_DacParam.AmpChan1*sin(ph+(2*i+1)*delta));
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
				if(i==(nsamples/2-1))*phase=ph+(2*i+2)*delta;
			}
		}
		if(g_DacParam.MaskChannels==3)
		{
			if(g_DacParam.SignalType==0)
			{
				for(int i=0;i<nsamples;i++)
				{
					low=(S32)(g_DacParam.AmpChan0*sin(ph+i*delta));
					hi=(S32)(g_DacParam.AmpChan1*sin(ph+i*delta));
					samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
					if(i==(nsamples-1))*phase=ph+(i+1)*delta;
				}
			}
			if(g_DacParam.SignalType==1)
			{
				for(int i=0;i<nsamples;i++)
				{
					low=(S32)(g_DacParam.AmpChan0*sin(ph+i*delta));
					hi=(S32)(g_DacParam.AmpChan1*cos(ph+i*delta));
					samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
					if(i==(nsamples-1))*phase=ph+(i+1)*delta;
				}
			}
		}
	}
//-------------------------------------------------------------------------------
	if(g_DacParam.SignalType==2)	//square
	{
		float amp,amp1;
		if(g_DacParam.MaskChannels==1)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				if(sin(ph+2*i*delta)>=0)amp=1; else amp=-1;
				if(sin(ph+(2*i+1)*delta)>=0)amp1=1; else amp1=-1;
				low=(S32)(g_DacParam.AmpChan0*amp);
				hi=(S32)(g_DacParam.AmpChan0*amp1);
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
				if(i==(nsamples/2-1))*phase=ph+(2*i+2)*delta;

			}
		}
		if(g_DacParam.MaskChannels==2)
		{
			for(int i=0;i<nsamples/2;i++)
			{
				if(sin(ph+2*i*delta)>=0)amp=1; else amp=-1;
				if(sin(ph+(2*i+1)*delta)>=0)amp1=1; else amp1=-1;
				low=(S32)(g_DacParam.AmpChan1*amp);
				hi=(S32)(g_DacParam.AmpChan1*amp1);
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
				if(i==(nsamples/2-1))*phase=ph+(2*i+2)*delta;
			}
		}
		if(g_DacParam.MaskChannels==3)
		{
			for(int i=0;i<nsamples;i++)
			{
				if(sin(ph+i*delta)>=0)amp=1; else amp=-1;
				low=(S32)(g_DacParam.AmpChan0*amp);
				hi=(S32)(g_DacParam.AmpChan1*amp);
				samples[i]=(0xffff & low) | (0xffff0000 & hi<<16);
				if(i==(nsamples-1))*phase=ph+(i+1)*delta;

			}
		}
	}
//------------------------------------------------------------------------------------
	if(g_DacParam.SignalType==2)	//square + Filter
	{	
		for(int	i=0;i<nsamples;i++)
		{
		
			low=0;
			hi=0;
			for(int j=0;j<g_DacParam.Filter;j++)
			{
				 low +=((samples[i+j]<<16)>>16)/g_DacParam.Filter;
				 hi +=(samples[i+j]>>16) /g_DacParam.Filter;
			}
			 samples[i]=(0xffff & low) | (0xffff0000 & (hi<<16));
		}
	}
return 1;
}
//*********************************************************************************************

//*********************************************************************************************
//         
//*********************************************************************************************
void CorrectOutFreq(void)
{
	double npi;
	double nnp;
	double delta=PI2*(double)g_DacParam.OutFreq/g_OutRate;
	npi=(delta*(double)g_ExamParam.SamplesPerChannel)/PI2;
	nnp =floor(npi+0.5);
	delta=PI2*(double)nnp/(double)g_ExamParam.SamplesPerChannel;
	double freq=floor(delta*g_OutRate/PI2+0.5);
	g_DacParam.OutFreq=freq;


}
//*********************************************************************************************
//   
//*********************************************************************************************
void ProgramSyncDac(int nmod, int invdclki)
{
	BRD_DacIntReg DacReg;
	BRD_DacSpec dacSpec;
	dacSpec.command=DAC216X400Mcmd_SETINTREG;
	dacSpec.arg= (PVOID)&DacReg; 

	DacReg.addr=0x8002;//!!!!
	if(invdclki==0)DacReg.data=0x04;//!!!!
	else DacReg.data=0x24;
	status = BRD_ctrl(g_hCaptSrv[nmod], 0, BRDctrl_DAC_SETSPECIFIC,  &dacSpec); //   

	DacReg.addr=0x8002;//!!!!
	if(invdclki==0)DacReg.data=0x02;//!!!!
	else DacReg.data=0x22;
	status = BRD_ctrl(g_hCaptSrv[nmod], 0, BRDctrl_DAC_SETSPECIFIC,  &dacSpec); //   

}
//*********************************************************************************************
//          FIFO
//*********************************************************************************************
S32 FifoOutputCPU(PVOID pSig, ULONG bBufSize)
{

	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	U32		statusFIFO=0x4;

	BRD_DataBuf data_buf;
	data_buf.pData = pSig;
	data_buf.size = bBufSize;
	for(int i=g_nCaptSrv-1;i>=0;i--)
	{
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_PUTDATA, &data_buf);

		if(i==0)status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_OUTSYNC,  NULL); //   (Master!)
		ProgramSyncDac(i,g_DacParam.INVDCLKI);
		
		Enable = 1;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
	}
	statusFIFO=0x4;
	for(int i=0;i<g_nCaptSrv;i++)
	{
	    while((statusFIFO & (1<<2))==(1<<2))//      FIFO  
		{
			 status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFOSTATUS, &statusFIFO); 
		}
	}
	Enable = 0;
	for(int i=0;i<g_nCaptSrv;i++)
	{
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
	}
	return 1;
}
//*****************************************************************************************************
//     FIFO    (restart/retransmit)    FIFO  
//*****************************************************************************************************
S32 FifoOutputCPUCycleFIFO(PVOID pSig, ULONG bBufSize,S32 cycle)
{

	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	BRD_DataBuf data_buf;
	data_buf.pData = pSig;
	data_buf.size = bBufSize;
	ULONG cycling = cycle; // 0-restart,1-retransmit

	for(int i=g_nCaptSrv-1;i>=0;i--)
	{
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETCYCLMODE, &cycling);
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_PUTDATA, &data_buf);

		if(i==0)status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_OUTSYNC,  NULL); //   (Master!)
		ProgramSyncDac(i,g_DacParam.INVDCLKI);

		Enable = 1;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
	}
	return 1;
}

//****************************************************************************************
//         FIFO - 
//****************************************************************************************
S32 FifoOutputDMA(void)
{
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	int fl_break = 0;
	U32		statusFIFO=0x4;
	BRDctrl_StreamCBufStart start_pars;
	ULONG flag = BRDstrm_DRQ_HALF; 

	for(int i=g_nCaptSrv-1;i>=0;i--)
	{
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_SETDRQ, &flag);
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE,  &Enable); //  
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 
//	status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_RESETFIFO, NULL);
		start_pars.isCycle = 0; //   
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_START, &start_pars); //  

		ULONG msTimeout = 1000; //       1 .
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_WAITBLOCK, &msTimeout);
		if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
		{	//   -
			printf("\nTIME-OUT! \n");
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_STOP, NULL); //  
			return 0;
		}

		if(i==0)status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_OUTSYNC,  NULL); //   (Master!)
		ProgramSyncDac(i,g_DacParam.INVDCLKI);

		Enable=1;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE,  &Enable); //  

	}
	for(int i=0;i<g_nCaptSrv;i++)
	{
		statusFIFO=0x4;
		while((statusFIFO & (1<<2))==(1<<2))//      FIFO 
		{
		   status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFOSTATUS, &statusFIFO); 
		}
	}
	Enable = 0;
	for(int i=0;i<g_nCaptSrv;i++)
	{
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE,  &Enable); //  
	}
	return 1;

}
//***************************************************************************************************

//*********************************************************************************************
//       SDRAM   - 
//*********************************************************************************************
S32 SdramWriteDMA(void)
{
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	ULONG cycling = 0; 
	ULONG flag = BRDstrm_DRQ_HALF; 
	U32		statusFIFO=0x4;

	ULONG msTimeout = 20000; //   .
	int fl_break = 0;
	
	for(int i=0;i<g_nCaptSrv;i++)
	{
		
		SdramModulWriteDMA(i);
		
	}
	
	return 1;
}
//*********************************************************************************************
//       SDRAM   - 
//*********************************************************************************************
S32 SdramModulWriteDMA(int nmod)
{
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	ULONG cycling = 0; 
	ULONG flag = BRDstrm_DRQ_HALF; 
	U32		statusFIFO=0x4;
	double phase=0;
	ULONG msTimeout = 20000; //   .
	int fl_break = 0;
	int nwrite=g_OutBufSize/g_SdramWriteBufSize;
	BRDctrl_StreamCBufStart start_pars;

		// set flag request DMA
	status = BRD_ctrl(g_hCaptSrv[nmod], 0, BRDctrl_STREAM_SETDRQ, &flag);
	start_pars.isCycle = 0; // no cycle 
	status = BRD_ctrl(g_hCaptSrv[nmod], 0, BRDctrl_SDRAM_FIFORESET, NULL);  //   FIFO SDRAM
	printf("nwrite= %d\n",nwrite);
	for(int i=0;i<nwrite;i++)
	{
		CalcSignalSDRAM(g_pbufSdram[nmod], g_SdramWriteBufSize/(g_NumChans*2),&phase);
		status = BRD_ctrl(g_hCaptSrv[nmod], 0, BRDctrl_STREAM_CBUF_START, &start_pars); //  
	    fl_break = 0;
		msTimeout = 20000; //   .
		status = BRD_ctrl(g_hCaptSrv[nmod], 0, BRDctrl_STREAM_CBUF_WAITBUF, &msTimeout);
		if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
		{	
			printf("TIME-OUT! \n");
			status = BRD_ctrl(g_hCaptSrv[nmod], 0, BRDctrl_STREAM_CBUF_STOP, NULL); //  
			return  0;
		}
		 if(_kbhit())return 1;


	printf("\rwrite = %d",i+1);

	}
	printf("\n");

	return 1;
}//*********************************************************************************************
//      SDRAM   - 
//*********************************************************************************************
S32 SdramOutToDAC(void)
{		
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	ULONG cycling =0;
	U32		statusFIFO=0x4;
	for(int i=g_nCaptSrv-1;i>=0;i--)
	{
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETCYCLMODE, &cycling);
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 

		if(i==0)status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_OUTSYNC,  NULL); //   (Master!)
		ProgramSyncDac(i,g_DacParam.INVDCLKI);

		Enable = 1;
	    status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_ENABLE, &Enable);    //     SDRAM
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable);	  //  
	}
	for(int i=0; i<g_nCaptSrv;i++)
	{
		//     SDRAM
		do {
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_ISACQCOMPLETE, &Status);
		} while(!Status);

		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_FLAGCLR, NULL);
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_ENABLE, &Enable); //    SDRAM
	}
	for(int i=0; i<g_nCaptSrv;i++)
	{
		statusFIFO=0x4;
		while((statusFIFO & (1<<2))==(1<<2))//      FIFO 
		{
		   status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFOSTATUS, &statusFIFO); 
		}
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable);   //   
	}
	return 1;
}
//***************************************************************************************************

//***************************************************************************************************
//     SDRAM    (restart/retransmit)     SDRAM 
//***************************************************************************************************
S32 CycleSdramOutput(S32 cycle)
{
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	ULONG cycling = cycle; // 0-restart,1-retransmit 
	ULONG flag = BRDstrm_DRQ_HALF;
	ULONG msTimeout = 20000; 
	BRDctrl_StreamCBufStart start_pars;
	for(int i=g_nCaptSrv-1;i>=0;i--)
	{
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 

		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_SETCYCLMODE, &cycling);
		// set flag request DMA
/*		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_SETDRQ, &flag);
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_FIFORESET, NULL);  //   FIFO SDRAM
		start_pars.isCycle = 0; //   
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_START, &start_pars); //  
		msTimeout = 20000; //   .
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_WAITBUF, &msTimeout);
		if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
		{	
			printf("TIME-OUT! \n");
			status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_STOP, NULL); //  
			return  0;
		}
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_STOP, NULL); //  
*/
		Enable = 0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable); //   
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 

		if(i==0)status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_OUTSYNC,  NULL); //   (Master!)
		ProgramSyncDac(i,g_DacParam.INVDCLKI);

		Enable = 1;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_SDRAM_ENABLE, &Enable);    //     SDRAM
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE, &Enable);	  //  
	}

	return 1;
}


//*********************************************************************************************
//  ()       FIFO  - 
//*********************************************************************************************
S32 FifoOutputCycleDMA(void)
{

	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;
	U32 statusFIFO;
	BRDctrl_StreamCBufStart start_pars;
	start_pars.isCycle = 1; //  
	ULONG msTimeout = 20000; 
	int	nblock=0; 
	ULONG flag = BRDstrm_DRQ_HALF; 

	for(int i=g_nCaptSrv-1;i>=0;i--)
	{
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_SETDRQ, &flag);
		Enable=0;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE,  &Enable); //  
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 
//	status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_RESETFIFO, NULL);

		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_STREAM_CBUF_START, &start_pars); //  

		if(i==0)status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_OUTSYNC,  NULL); //   (Master!)
		ProgramSyncDac(i,g_DacParam.INVDCLKI);

		Enable=1;
		status = BRD_ctrl(g_hCaptSrv[i], 0, BRDctrl_DAC_ENABLE,  &Enable); //  
	}
		msTimeout = 2000; //       20 .
	while(!_kbhit())
	{
		status = BRD_ctrl(g_hCaptSrv[0], 0, BRDctrl_STREAM_CBUF_WAITBLOCK, &msTimeout);
		if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
		{	//   -
			printf("\nTIME-OUT! \n");
			status = BRD_ctrl(g_hCaptSrv[0], 0, BRDctrl_DAC_FIFOSTATUS, &statusFIFO); 

			return 1;
		}
		status = BRD_ctrl(g_hCaptSrv[0], 0, BRDctrl_DAC_FIFOSTATUS, &statusFIFO); 
		if((statusFIFO>>8)&1)
		{
			printf("\nUnderflow! \n");
			printf("Press any key for stop!\n");

			_getch();
			return 0;

		}
		nblock++;
		printf("\rBlock %d",nblock);
	}
	_getch();
	Enable=0;

	return 0;

}
//*********************************************************************************************
bool SetParamSDRAM(BRD_Handle hSrv)
{
	
		S32	status;
		BRD_SdramCfg SdramConfig;
		ULONG PhysMemSize;
		status = BRD_ctrl(hSrv, 0, BRDctrl_SDRAM_GETCFG, &SdramConfig);
		if(status < 0)
		{
			PhysMemSize = 0; // SDRAM 
			return false;
		}
		else
		{					// SDRAM 
			PhysMemSize =	(1 << SdramConfig.RowAddrBits) *
							(1 << SdramConfig.ColAddrBits) * 
							SdramConfig.ModuleBanks * 
							SdramConfig.ChipBanks *
							SdramConfig.ModuleCnt * 2; // size (U32)
		
			printf("SDRAM total size = %d Mbytes\n",(PhysMemSize/(1024*1024))*4);
 
			__int64 BytesPhysMemSize=(__int64)PhysMemSize*4;
			ULONG addr = 0;
			status = BRD_ctrl(hSrv, 0, BRDctrl_SDRAM_SETSTARTADDR, &addr); // set address for write
			status = BRD_ctrl(hSrv, 0, BRDctrl_SDRAM_SETREADADDR, &addr);  // set address for read
			ULONG mem_size =(ULONG) (g_OutBufSize >> 2); // set size activ zone (U32)
			status = BRD_ctrl(hSrv, 0, BRDctrl_SDRAM_SETMEMSIZE, &mem_size);
			__int64 OutBufSize =((__int64) mem_size) << 2;		 // get size activ zone (bytes)
			if(OutBufSize>BytesPhysMemSize)OutBufSize=BytesPhysMemSize;
			if(g_OutBufSize>OutBufSize)g_OutBufSize=OutBufSize;

			if(BRD_errcmp(status, BRDerr_OK))
			{
				return true;
			}
			else
			{
				DisplayError(status, "BRDctrl_SDRAM_SETMEMSIZE");
				return false;
			}
		 }
	return true;

}

//************************************************************************************************
//=*********************************************************************************
//						  
//=*********************************************************************************
bool    ReadIniFile( TCHAR *pFileName )
{
	TCHAR Buffer[128];
	TCHAR *endptr;
	TCHAR iniFilePath[MAX_PATH];
	FILE    *stream;

	GetCurrentDirectory(sizeof(iniFilePath)/sizeof(TCHAR), iniFilePath);

	_tcscat_s(iniFilePath, pFileName);
	stream = fopen(iniFilePath, "rt" );
    if( stream == NULL)return false;
	else     fclose(stream);

    for(int i=0; i<sizeof(iniEntry)/sizeof(iniEntry[0]); i++ )
	{
		GetPrivateProfileString(iniEntry[i].section, iniEntry[i].title, "0", Buffer, sizeof(Buffer), iniFilePath);
		switch(iniEntry[i].format){
			case 0:	
				{
					int     base = 10;
					char    *s = Buffer;
					while (isspace(*s)) s++;
                    if( (s[0]=='0') && ((s[1]=='x') || (s[1]=='X')) )
                    base = 16;
                    *(int*)iniEntry[i].ptr = strtoul( Buffer, &endptr, base );
				}
				break;
			case 1:	*(float*)iniEntry[i].ptr=atof(Buffer);
				break;
			case 2:	*(double*)iniEntry[i].ptr=atof(Buffer);
				break;
			case 3:	 lstrcpy((TCHAR*)iniEntry[i].ptr,Buffer);
				break;

		}
	}
	return true;
}

//
// End of file
//
